// Floor Pivot Points
// Author: cvax
// Version: v1.1
//
// Changelog
// v1.1 - Added midpoint plots
// v1.0 - Public release

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

namespace NinjaTrader.Indicator
{
	[Description("Floor Pivot Points to R5/S5 by cvax")]
	[Gui.Design.DisplayName("Pivot Points (Floor)")]
	public class FloorPivots : Indicator
	{
		#region Variables
		private	SolidBrush[]	brushes			= { new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), 
													new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),
													new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), 
													new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),
													new SolidBrush(Color.Black)};
		private	double			currentClose	= 0;
		private DateTime		currentDate		= Cbi.Globals.MinDate;
		private DateTime		currentMonth	= Cbi.Globals.MinDate;
		private DateTime		currentWeek		= Cbi.Globals.MinDate;
		private	double			currentHigh		= 0;
		private	double			currentLow		= 0;
		private Data.PivotRange pivotRangeType	= PivotRange.Daily;
		private bool			labels			= true;
		private bool			floorlabels		= true;
		private bool			midpoint		= true;
		private	double			pp				= 0;
		private	double			r1				= 0;
		private	double			r2				= 0;
		private	double			r3				= 0;
		private	double			r4				= 0;
		private	double			r5				= 0;
		private	double			s1				= 0;
		private	double			s2				= 0;
		private	double			s3				= 0;
		private	double			s4				= 0;
		private	double			s5				= 0;
		private	StringFormat	stringFormat	= new StringFormat();
		private int				width			= 0;
		#endregion

		protected override void Initialize()
		{
			Add(new Plot(Color.Red,		"R5"));
			Add(new Plot(Color.Red,		"R4"));
			Add(new Plot(Color.Red,		"R3"));
			Add(new Plot(Color.Red,		"R2"));
			Add(new Plot(Color.Red,		"R1"));
			Add(new Plot(Color.DarkGreen,	"PP"));
			Add(new Plot(Color.Blue,	"S1"));
			Add(new Plot(Color.Blue,	"S2"));
			Add(new Plot(Color.Blue,	"S3"));
			Add(new Plot(Color.Blue,	"S4"));
			Add(new Plot(Color.Blue,	"S5"));
			
			if(midpoint == true)
			{
				Add(new Plot(Color.Gray,	"R5-R4 Midpoint"));
				Add(new Plot(Color.Gray,	"R4-R3 Midpoint"));
				Add(new Plot(Color.Gray,	"R3-R2 Midpoint"));
				Add(new Plot(Color.Gray,	"R2-R1 Midpoint"));
				Add(new Plot(Color.Gray,	"R1-PP Midpoint"));
				Add(new Plot(Color.Gray,	"PP-S1 Midpoint"));
				Add(new Plot(Color.Gray,	"S1-S2 Midpoint"));
				Add(new Plot(Color.Gray,	"S2-S3 Midpoint"));
				Add(new Plot(Color.Gray,	"S3-S4 Midpoint"));
				Add(new Plot(Color.Gray,	"S4-S5 Midpoint"));
			}
			
			Plots[0].Pen = new Pen(Color.Red, 2);
			Plots[1].Pen = new Pen(Color.Red, 2);
			Plots[2].Pen = new Pen(Color.Red, 2);
			Plots[3].Pen = new Pen(Color.Red, 2);
			Plots[4].Pen = new Pen(Color.Red, 2);
			Plots[5].Pen = new Pen(Color.Black, 2);
			Plots[6].Pen = new Pen(Color.Blue, 2);
			Plots[7].Pen = new Pen(Color.Blue, 2);
			Plots[8].Pen = new Pen(Color.Blue, 2);
			Plots[9].Pen = new Pen(Color.Blue, 2);
			Plots[10].Pen = new Pen(Color.Blue, 2);
			
			AutoScale				= false;
			CalculateOnBarClose		= false;
			Overlay					= true;
			stringFormat.Alignment	= StringAlignment.Far;
		}

		protected override void OnBarUpdate()
		{
			if (Bars == null)
				return; 
			if (!Session.IsIntraday(Bars.Period.Id) && Bars.Period.Id != PeriodType.Day)
				return;
			if (Bars.Period.Id == PeriodType.Day && pivotRangeType == PivotRange.Daily)
				return;
			if (Bars.Period.Id == PeriodType.Day && Bars.Period.Value > 1)
				return;

			if ((currentDate != Cbi.Globals.MinDate && pivotRangeType == PivotRange.Daily && Time[0].Date != currentDate)
				|| (currentWeek != Cbi.Globals.MinDate && pivotRangeType == PivotRange.Weekly && RoundUpTimeToPeriodTime(Time[0].Date, PivotRange.Weekly) != currentWeek) 
				|| (currentMonth != Cbi.Globals.MinDate && pivotRangeType == PivotRange.Monthly && RoundUpTimeToPeriodTime(Time[0].Date, PivotRange.Monthly) != currentMonth)) 
			{
				pp				= (currentHigh + currentLow + currentClose) / 3;
				s1				= 2 * pp - currentHigh;
				r1				= 2 * pp - currentLow;
				s2				= pp - (currentHigh - currentLow);
				r2				= pp + (currentHigh - currentLow);
				s3				= pp - 2*(currentHigh - currentLow);
				r3				= pp + 2*(currentHigh - currentLow);
				s4				= pp - 3*(currentHigh - currentLow);
				r4				= pp + 3*(currentHigh - currentLow);
				s5				= pp - 4*(currentHigh - currentLow);
				r5				= pp + 4*(currentHigh - currentLow);
			}
			if (pivotRangeType == PivotRange.Daily)
				currentDate = Time[0].Date;
			if (pivotRangeType == PivotRange.Weekly)
				currentWeek = RoundUpTimeToPeriodTime(Time[0].Date, PivotRange.Weekly);
			if (pivotRangeType == PivotRange.Monthly)
				currentMonth = RoundUpTimeToPeriodTime(Time[0].Date, PivotRange.Monthly);

			if (pp != 0)
			{
				PP.Set(pp);
				R1.Set(r1);
				S1.Set(s1);
				R2.Set(r2);
				S2.Set(s2);
				R3.Set(r3);
				S3.Set(s3);
				R4.Set(r4);
				S4.Set(s4);
				R5.Set(r5);
				S5.Set(s5);
			}
			if(midpoint)
			{
				FiftyR54.Set((r5-r4)*.50+r4);
				FiftyR43.Set((r4-r3)*.50+r3);
				FiftyR32.Set((r3-r2)*.50+r2);
				FiftyR21.Set((r2-r1)*.50+r1);
				FiftyR1PP.Set((r1-pp)*.50+pp);
				FiftyPPS1.Set((pp-s1)*.50+s1);
				FiftyS12.Set((s1-s2)*.50+s2);
				FiftyS23.Set((s2-s3)*.50+s3);
				FiftyS34.Set((s3-s4)*.50+s4);
				FiftyS45.Set((s4-s5)*.50+s5);
			}
		}

		#region Properties
		[Description("Type of period for pivot points.")]
		[Category("Parameters")]
		public Data.PivotRange PivotRangeType 
		{
			get { return pivotRangeType; }
			set { pivotRangeType = value; }
		}

		[Description("Turn Pivot Labels On/Off.")]
		[Category("Pivot Line Settings")]
		public bool Labels 
		{
			get { return labels; }
			set { labels = value; }
		}
		
		[Description("Differentiate labeling between Floor Pivots and regular Pivots")]
		[Category("Pivot Line Settings")]
		public bool FloorLabels 
		{
			get { return floorlabels; }
			set { floorlabels = value; }
		}
		
		[Description("# of bars back the Pivot Line will be drawn. (0=maximum width)")]
		[Category("Pivot Line Settings")]
		public int Width
		{
			get { return width; }
			set { width = Math.Max(0, value); }
		}
		
		[Description("Turn Midpoint Lines On/Off.")]
		[Category("Pivot Line Settings")]
		public bool MidPoint 
		{
			get { return midpoint; }
			set { midpoint = value; }
		}
		
		[Description("The Close value used for Pivots calculations.")]
		[Category("Parameters")]
		public double Close_Floor
		{
			get { return currentClose; }
			set { currentClose = Math.Max(0, value); }
		}
		
		[Description("The High value used for Pivots calculations.")]
		[Category("Parameters")]
		public double High_Floor
		{
			get { return currentHigh; }
			set { currentHigh = Math.Max(0, value); }
		}
		
		[Description("The Low value used for Pivots calculations.")]
		[Category("Parameters")]
		public double Low_Floor
		{
			get { return currentLow; }
			set { currentLow = Math.Max(0, value); }
		}

		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PP
		{
			get { return Values[5]; }
		}

		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R1
		{
			get { return Values[4]; }
		}

		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R2
		{
			get { return Values[3]; }
		}

		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R3
		{
			get { return Values[2]; }
		}
		
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R4
		{
			get { return Values[1]; }
		}
		
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R5
		{
			get { return Values[0]; }
		}
		
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S1
		{
			get { return Values[6]; }
		}
		
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S2
		{
			get { return Values[7]; }
		}
		
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S3
		{
			get { return Values[8]; }
		}
		
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S4
		{
			get { return Values[9]; }
		}
		
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S5
		{
			get { return Values[10]; }
		}
		
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries FiftyR54
		{
			get { return Values[11]; }
		}
		
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries FiftyR43
		{
			get { return Values[12]; }
		}
				
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries FiftyR32
		{
			get { return Values[13]; }
		}
				
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries FiftyR21
		{
			get { return Values[14]; }
		}
				
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries FiftyR1PP
		{
			get { return Values[15]; }
		}
				
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries FiftyPPS1
		{
			get { return Values[16]; }
		}
				
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries FiftyS12
		{
			get { return Values[17]; }
		}
				
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries FiftyS23
		{
			get { return Values[18]; }
		}
				
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries FiftyS34
		{
			get { return Values[19]; }
		}
				
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries FiftyS45
		{
			get { return Values[20]; }
		}
		#endregion

		#region Miscellaneous
		private int GetDayOfWeek(DateTime date)
		{
			DateTime saturday = new DateTime(1776, 7, 4).AddDays(2);
			TimeSpan diff = date.Subtract(saturday);
			return (diff.Days % 7);
		}

		private DateTime RoundUpTimeToPeriodTime(DateTime time, PivotRange pivotRange)
		{
			if (pivotRange == PivotRange.Weekly)
			{
				DateTime periodStart = time.AddDays((6-GetDayOfWeek(time)));
				return periodStart.Date.AddDays(System.Math.Ceiling(System.Math.Ceiling(time.Date.Subtract(periodStart.Date).TotalDays) / 7) * 7).Date;
			}
			else if (pivotRange == PivotRange.Monthly)
			{
				DateTime result = new DateTime(time.Year, time.Month, 1); 
				return result.AddMonths(1).AddDays(-1);
			}
			else
				return time;
		}

		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			if (Bars == null || ChartControl == null)
				return;

			int	barWidth = ChartControl.ChartStyle.GetBarPaintWidth(ChartControl.BarWidth);
			for (int seriesCount = 0; seriesCount < Values.Length; seriesCount++)
			{
				SolidBrush		brush				= brushes[seriesCount];
				DateTime		lastDate			= Cbi.Globals.MinDate;
				DateTime		lastWeek			= Cbi.Globals.MinDate;
				DateTime		lastMonth			= Cbi.Globals.MinDate;
				int				firstX				= -1;
				int				lastX				= -1;
				int				lastY				= -1;
				int				xcounter			= 0;
				SmoothingMode	oldSmoothingMode	= graphics.SmoothingMode;
				GraphicsPath	path				= new GraphicsPath();
				Gui.Chart.Plot	plot				= Plots[seriesCount];
				DataSeries	series				= (DataSeries) Values[seriesCount];
				
				if (brush.Color != plot.Pen.Color)	
					brush = new SolidBrush(plot.Pen.Color);
				
				if(Width==0)
					Width=ChartControl.BarsPainted;

				for (int count = ChartControl.BarsPainted - 1; count >= ChartControl.BarsPainted - Math.Min(ChartControl.BarsPainted, Width); count--)
				{
					
					int idx = ChartControl.LastBarPainted - ChartControl.BarsPainted + 1 + count;
					if (idx - Displacement < 0 || idx - Displacement >= Bars.Count || (!ChartControl.ShowBarsRequired && idx - Displacement < BarsRequired))
						continue;
					else if (!series.IsValidPlot(idx))
						continue;

					if (pivotRangeType == PivotRange.Daily && lastDate == Cbi.Globals.MinDate) 
						lastDate = Bars.Get(idx).Time.Date;
					else if (pivotRangeType == PivotRange.Weekly && lastWeek == Cbi.Globals.MinDate) 
						lastWeek = RoundUpTimeToPeriodTime(Bars.Get(idx).Time.Date, PivotRange.Weekly);
					else if (pivotRangeType == PivotRange.Monthly && lastMonth == Cbi.Globals.MinDate) 
						lastMonth = RoundUpTimeToPeriodTime(Bars.Get(idx).Time.Date, PivotRange.Monthly);
					else if (pivotRangeType == PivotRange.Daily && lastDate != Bars.Get(idx).Time.Date 
							|| pivotRangeType == PivotRange.Weekly && lastWeek != RoundUpTimeToPeriodTime(Bars.Get(idx).Time.Date, PivotRange.Weekly)
							|| pivotRangeType == PivotRange.Monthly && lastMonth != RoundUpTimeToPeriodTime(Bars.Get(idx).Time.Date, PivotRange.Monthly))
						continue;

					double		val = series.Get(idx);
					int			x	= (int) (ChartControl.CanvasRight - ChartControl.BarMarginRight - barWidth / 2
										- (ChartControl.BarsPainted - 1) * ChartControl.BarSpace + count * ChartControl.BarSpace);
					int			y	= (int) ((bounds.Y + bounds.Height) - ((val - min ) / Gui.Chart.ChartControl.MaxMinusMin(max, min)) * bounds.Height);

					if (xcounter == 0)
					{
						firstX	= x;
					}
					if (lastX >= 0)
					{
						if (y != lastY) // Problem here is, that last bar of old day has date of new day
							y = lastY;
						path.AddLine(lastX - plot.Pen.Width / 2, lastY, x - plot.Pen.Width / 2, y);
					}
					lastX	= x;
					lastY	= y;
					
					xcounter++;
				}

				graphics.SmoothingMode = SmoothingMode.AntiAlias;
				graphics.DrawPath(plot.Pen, path);
				graphics.SmoothingMode = oldSmoothingMode;
				if ( labels == true )
				{
					if (pivotRangeType == PivotRange.Daily)
					{
						if (floorlabels)
						{
							graphics.DrawString("Floor " + plot.Name, ChartControl.Font, brush, firstX, lastY - ChartControl.Font.GetHeight(), stringFormat);			//align label right
							//graphics.DrawString("Floor " + plot.Name, ChartControl.Font, brush, lastX, lastY - ChartControl.Font.GetHeight() / 2, stringFormat);		//align label left
						}
						else
						{
							graphics.DrawString(plot.Name, ChartControl.Font, brush, firstX, lastY - ChartControl.Font.GetHeight(), stringFormat);			//align label right
							//graphics.DrawString(plot.Name, ChartControl.Font, brush, lastX, lastY - ChartControl.Font.GetHeight() / 2, stringFormat);		//align label left
						}
					}
					else if (pivotRangeType == PivotRange.Weekly)
					{
						if (floorlabels)
						{
							graphics.DrawString("Weekly Floor " + plot.Name, ChartControl.Font, brush, firstX, lastY - ChartControl.Font.GetHeight(), stringFormat);			//align label right
							//graphics.DrawString("Weekly Floor" + plot.Name, ChartControl.Font, brush, lastX, lastY - ChartControl.Font.GetHeight() / 2, stringFormat);		//align label left
						}
						else
						{
							graphics.DrawString("Weekly " + plot.Name, ChartControl.Font, brush, firstX, lastY - ChartControl.Font.GetHeight(), stringFormat);			//align label right
							//graphics.DrawString("Weekly " + plot.Name, ChartControl.Font, brush, lastX, lastY - ChartControl.Font.GetHeight() / 2, stringFormat);		//align label left
						}
					}
					else
					{
						if (floorlabels)
						{
							graphics.DrawString("Monthly Floor " + plot.Name, ChartControl.Font, brush, firstX, lastY - ChartControl.Font.GetHeight(), stringFormat);			//align label right
							//graphics.DrawString("Monthly Floor " + plot.Name, ChartControl.Font, brush, lastX, lastY - ChartControl.Font.GetHeight() / 2, stringFormat);		//align label left
						}
						else
						{
							graphics.DrawString("Monthly " + plot.Name, ChartControl.Font, brush, firstX, lastY - ChartControl.Font.GetHeight(), stringFormat);			//align label right
							//graphics.DrawString("Monthly " + plot.Name, ChartControl.Font, brush, lastX, lastY - ChartControl.Font.GetHeight() / 2, stringFormat);		//align label left
						}
					}
				}
				else
				{
					graphics.DrawString("", ChartControl.Font, brush, firstX, lastY - ChartControl.Font.GetHeight(), stringFormat);			//align label right
					//graphics.DrawString("", ChartControl.Font, brush, lastX, lastY - ChartControl.Font.GetHeight() / 2, stringFormat);		//align label left
				}
			}
		}
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private FloorPivots[] cacheFloorPivots = null;

        private static FloorPivots checkFloorPivots = new FloorPivots();

        /// <summary>
        /// Floor Pivot Points to R5/S5 by cvax
        /// </summary>
        /// <returns></returns>
        public FloorPivots FloorPivots(double close_Floor, double high_Floor, double low_Floor, Data.PivotRange pivotRangeType)
        {
            return FloorPivots(Input, close_Floor, high_Floor, low_Floor, pivotRangeType);
        }

        /// <summary>
        /// Floor Pivot Points to R5/S5 by cvax
        /// </summary>
        /// <returns></returns>
        public FloorPivots FloorPivots(Data.IDataSeries input, double close_Floor, double high_Floor, double low_Floor, Data.PivotRange pivotRangeType)
        {
            checkFloorPivots.Close_Floor = close_Floor;
            close_Floor = checkFloorPivots.Close_Floor;
            checkFloorPivots.High_Floor = high_Floor;
            high_Floor = checkFloorPivots.High_Floor;
            checkFloorPivots.Low_Floor = low_Floor;
            low_Floor = checkFloorPivots.Low_Floor;
            checkFloorPivots.PivotRangeType = pivotRangeType;
            pivotRangeType = checkFloorPivots.PivotRangeType;

            if (cacheFloorPivots != null)
                for (int idx = 0; idx < cacheFloorPivots.Length; idx++)
                    if (Math.Abs(cacheFloorPivots[idx].Close_Floor - close_Floor) <= double.Epsilon && Math.Abs(cacheFloorPivots[idx].High_Floor - high_Floor) <= double.Epsilon && Math.Abs(cacheFloorPivots[idx].Low_Floor - low_Floor) <= double.Epsilon && cacheFloorPivots[idx].PivotRangeType == pivotRangeType && cacheFloorPivots[idx].EqualsInput(input))
                        return cacheFloorPivots[idx];

            FloorPivots indicator = new FloorPivots();
            indicator.SetUp();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Close_Floor = close_Floor;
            indicator.High_Floor = high_Floor;
            indicator.Low_Floor = low_Floor;
            indicator.PivotRangeType = pivotRangeType;

            FloorPivots[] tmp = new FloorPivots[cacheFloorPivots == null ? 1 : cacheFloorPivots.Length + 1];
            if (cacheFloorPivots != null)
                cacheFloorPivots.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheFloorPivots = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Floor Pivot Points to R5/S5 by cvax
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.FloorPivots FloorPivots(double close_Floor, double high_Floor, double low_Floor, Data.PivotRange pivotRangeType)
        {
            return _indicator.FloorPivots(Input, close_Floor, high_Floor, low_Floor, pivotRangeType);
        }

        /// <summary>
        /// Floor Pivot Points to R5/S5 by cvax
        /// </summary>
        /// <returns></returns>
        public Indicator.FloorPivots FloorPivots(Data.IDataSeries input, double close_Floor, double high_Floor, double low_Floor, Data.PivotRange pivotRangeType)
        {
            return _indicator.FloorPivots(input, close_Floor, high_Floor, low_Floor, pivotRangeType);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Floor Pivot Points to R5/S5 by cvax
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.FloorPivots FloorPivots(double close_Floor, double high_Floor, double low_Floor, Data.PivotRange pivotRangeType)
        {
            return _indicator.FloorPivots(Input, close_Floor, high_Floor, low_Floor, pivotRangeType);
        }

        /// <summary>
        /// Floor Pivot Points to R5/S5 by cvax
        /// </summary>
        /// <returns></returns>
        public Indicator.FloorPivots FloorPivots(Data.IDataSeries input, double close_Floor, double high_Floor, double low_Floor, Data.PivotRange pivotRangeType)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.FloorPivots(input, close_Floor, high_Floor, low_Floor, pivotRangeType);
        }

    }
}
#endregion
